@@ -1,3 +1,4 @@
+/*-*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
 An indicator to time and date related information in the menubar.
 
@@ -30,7 +31,7 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <math.h>
 #include <gconf/gconf-client.h>
 
-#include <libdbusmenu-gtk/menuitem.h>
+#include <libdbusmenu-gtk3/menuitem.h>
 #include <libdbusmenu-glib/server.h>
 #include <libdbusmenu-glib/client.h>
 #include <libdbusmenu-glib/menuitem.h>
@@ -60,6 +61,7 @@ static gboolean update_timezone_menu_items(gpointer user_data);
 static void setup_timer (void);
 static void geo_client_invalid (GeoclueMasterClient * client, gpointer user_data);
 static void geo_address_change (GeoclueMasterClient * client, gchar * a, gchar * b, gchar * c, gchar * d, gpointer user_data);
+static gboolean get_greeter_mode (void);
 
 static IndicatorService * service = NULL;
 static GMainLoop * mainloop = NULL;
@@ -221,29 +223,15 @@ update_current_timezone (void) {
 		current_timezone = NULL;
 	}
 
-	GError * error = NULL;
-	gchar * tempzone = NULL;
-	if (!g_file_get_contents(TIMEZONE_FILE, &tempzone, NULL, &error)) {
-		g_warning("Unable to read timezone file '" TIMEZONE_FILE "': %s", error->message);
-		g_error_free(error);
+	current_timezone = read_timezone ();
+	if (current_timezone == NULL) {
 		return;
 	}
 
-	/* This shouldn't happen, so let's make it a big boom! */
-	g_return_if_fail(tempzone != NULL);
-
-	/* Note: this really makes sense as strstrip works in place
-	   so we end up with something a little odd without the dup
-	   so we have the dup to make sure everything is as expected
-	   for everyone else. */
-	current_timezone = g_strdup(g_strstrip(tempzone));
-	g_free(tempzone);
-
 	g_debug("System timezone is: %s", current_timezone);
 
 	check_timezone_sync();
 
-    if (error != NULL) g_error_free(error);
 	return;
 }
 
@@ -276,10 +264,8 @@ quick_set_tz_proxy_cb (GObject *object, GAsyncResult *res, gpointer zone)
 		return;
 	}
 
-	gchar * file = g_build_filename ("/usr/share/zoneinfo", (char *)zone, NULL);
-	g_dbus_proxy_call (proxy, "SetTimezone", g_variant_new ("(s)", file),
+	g_dbus_proxy_call (proxy, "SetTimezone", g_variant_new ("(s)", zone),
 	                   G_DBUS_CALL_FLAGS_NONE, -1, NULL, quick_set_tz_cb, NULL);
-	g_free (file);
 	g_free (zone);
 	g_object_unref (proxy);
 }
@@ -314,25 +300,25 @@ quick_set_tz (DbusmenuMenuitem * menuitem, guint timestamp, gpointer user_data)
 static gboolean
 update_datetime (gpointer user_data)
 {
+	GDateTime *datetime;
+	gchar *utf8;
+
 	g_debug("Updating Date/Time");
 
-	gchar longstr[128];
-	time_t t;
-	struct tm *ltime;
-
-	t = time(NULL);
-	ltime = localtime(&t);
-	if (ltime == NULL) {
+	datetime = g_date_time_new_now_local ();
+	if (datetime == NULL) {
 		g_warning("Error getting local time");
 		dbusmenu_menuitem_property_set(date, DBUSMENU_MENUITEM_PROP_LABEL, _("Error getting time"));
+		g_date_time_unref (datetime);
 		return FALSE;
 	}
 
-	/* Translators: strftime(3) style date format on top of the menu when you click on the clock */
-	strftime(longstr, 128, _("%A, %e %B %Y"), ltime);
-	
-	gchar * utf8 = g_locale_to_utf8(longstr, -1, NULL, NULL, NULL);
+	/* eranslators: strftime(3) style date format on top of the menu when you click on the clock */
+        utf8 = g_date_time_format (datetime, _("%A, %e %B %Y"));
+
 	dbusmenu_menuitem_property_set(date, DBUSMENU_MENUITEM_PROP_LABEL, utf8);
+
+	g_date_time_unref (datetime);
 	g_free(utf8);
 
 	return FALSE;
@@ -445,7 +431,10 @@ static guint ecaltimer = 0;
 static void
 start_ecal_timer(void)
 {
-	if (ecaltimer != 0) g_source_remove(ecaltimer);
+	if (ecaltimer != 0) {
+		g_source_remove(ecaltimer);
+		ecaltimer = 0;
+	}
 	if (update_appointment_menu_items(NULL))
 		ecaltimer = g_timeout_add_seconds(60*5, update_appointment_menu_items, NULL); 	
 }
@@ -453,7 +442,10 @@ start_ecal_timer(void)
 static void
 stop_ecal_timer(void)
 {
-	if (ecaltimer != 0) g_source_remove(ecaltimer);
+	if (ecaltimer != 0) {
+		g_source_remove(ecaltimer);
+		ecaltimer = 0;
+	}
 }
 static gboolean
 idle_start_ecal_timer (gpointer data)
@@ -497,7 +489,7 @@ check_for_calendar (gpointer user_data)
 	dbusmenu_menuitem_property_set_bool(date, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
 	
 	gchar *evo = g_find_program_in_path("evolution");
-	if (evo != NULL) {
+	if (!get_greeter_mode () && evo != NULL) {
 		g_debug("Found the calendar application: %s", evo);
 		
 		g_signal_connect (G_OBJECT(date), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
@@ -507,7 +499,7 @@ check_for_calendar (gpointer user_data)
 		dbusmenu_menuitem_property_set(events_separator, DBUSMENU_MENUITEM_PROP_TYPE, DBUSMENU_CLIENT_TYPES_SEPARATOR);
 		dbusmenu_menuitem_child_add_position(root, events_separator, 2);
 		add_appointment = dbusmenu_menuitem_new();
-		dbusmenu_menuitem_property_set (add_appointment, DBUSMENU_MENUITEM_PROP_LABEL, _("Add Event..."));
+		dbusmenu_menuitem_property_set (add_appointment, DBUSMENU_MENUITEM_PROP_LABEL, _("Add Event…"));
 		dbusmenu_menuitem_property_set_bool(add_appointment, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
 		g_signal_connect(G_OBJECT(add_appointment), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(activate_cb), "evolution -c calendar");
 		dbusmenu_menuitem_child_add_position (root, add_appointment, 3);
@@ -550,6 +542,10 @@ static gboolean
 update_timezone_menu_items(gpointer user_data) {
 	g_debug("Updating timezone menu items");
 
+	if (locations_separator == NULL || current_location == NULL) {
+		return FALSE;
+	}
+
 	gchar ** locations = g_settings_get_strv(conf, SETTINGS_LOCATIONS_S);
 
 	if (locations == NULL) { 
@@ -712,7 +708,7 @@ update_appointment_menu_items (gpointer user_data)
 
 	// Get today & work out query times
 	time(&curtime);
-  	struct tm *today = localtime(&curtime);
+	struct tm *today = localtime(&curtime);
 	const int mday = today->tm_mday;
 	const int mon = today->tm_mon;
 	const int year = today->tm_year;
@@ -759,9 +755,10 @@ update_appointment_menu_items (gpointer user_data)
 		for (l = comp_instances; l; l = l->next) {
 			const struct comp_instance *ci = l->data;
 			g_object_unref(ci->comp);
-			g_list_free(comp_instances);
-			comp_instances = NULL;
 		}
+		g_list_free(comp_instances);
+		comp_instances = NULL;
+
 	}
 	GSList *cal_list = gconf_client_get_list(gconf, "/apps/evolution/calendar/display/selected_calendars", GCONF_VALUE_STRING, &gerror);
 	if (gerror) {
@@ -770,6 +767,7 @@ update_appointment_menu_items (gpointer user_data)
 	  gerror = NULL;
 	  cal_list = NULL;
 	}
+	
 	// Generate instances for all sources
 	for (g = e_source_list_peek_groups (sources); g; g = g->next) {
 		ESourceGroup *group = E_SOURCE_GROUP (g->data);
@@ -781,12 +779,26 @@ update_appointment_menu_items (gpointer user_data)
 			ECal *ecal = e_cal_new(source, E_CAL_SOURCE_TYPE_EVENT);
 			e_cal_set_auth_func (ecal, (ECalAuthFunc) auth_func, NULL);
 			
+			icaltimezone* current_zone = icaltimezone_get_builtin_timezone(current_timezone);
+			if (!current_zone) {
+				// current_timezone may be a TZID?
+				current_zone = icaltimezone_get_builtin_timezone_from_tzid(current_timezone);
+			}
+			if (current_zone && !e_cal_set_default_timezone(ecal, current_zone, &gerror)) {
+				g_debug("Failed to set ecal default timezone %s", gerror->message);
+				g_error_free(gerror);
+				gerror = NULL;
+				g_object_unref(ecal);
+				continue;
+			}
+			
 			if (!e_cal_open(ecal, FALSE, &gerror)) {
 				g_debug("Failed to get ecal sources %s", gerror->message);
 				g_error_free(gerror);
 				gerror = NULL;
+				g_object_unref(ecal);
 				continue;
-        	}
+			}
 			const gchar *ecal_uid = e_source_peek_uid(source);
 			gboolean match = FALSE;
 			g_debug("Checking ecal_uid is enabled: %s", ecal_uid);
@@ -797,10 +809,14 @@ update_appointment_menu_items (gpointer user_data)
 					break;
 				}
 			}
-			if (!match) continue;
+			if (!match) {
+				g_object_unref(ecal);
+				continue;
+			}
 			g_debug("ecal_uid is enabled, generating instances");
 			
 			e_cal_generate_instances (ecal, t1, t2, (ECalRecurInstanceFn) populate_appointment_instances, (gpointer) source);
+			g_object_unref(ecal);
 		}
 	}
 	g_debug("Number of ECalComponents returned: %d", g_list_length(comp_instances));
@@ -1040,13 +1056,13 @@ check_for_timeadmin (gpointer user_data)
 {
 	g_return_val_if_fail (settings != NULL, FALSE);
 
-	gchar * timeadmin = g_find_program_in_path("indicator-datetime-preferences");
+	gchar * timeadmin = g_find_program_in_path("gnome-control-center");
 	if (timeadmin != NULL) {
-		g_debug("Found the indicator-datetime-preferences application: %s", timeadmin);
+		g_debug("Found the gnome-control-center application: %s", timeadmin);
 		dbusmenu_menuitem_property_set_bool(settings, DBUSMENU_MENUITEM_PROP_ENABLED, TRUE);
 		g_free(timeadmin);
 	} else {
-		g_debug("Unable to find indicator-datetime-preferences app.");
+		g_debug("Unable to find gnome-control-center app.");
 		dbusmenu_menuitem_property_set_bool(settings, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
 	}
 
@@ -1075,7 +1091,7 @@ build_menus (DbusmenuMenuitem * root)
 	g_debug("Building Menus.");
 	if (date == NULL) {
 		date = dbusmenu_menuitem_new();
-		dbusmenu_menuitem_property_set     (date, DBUSMENU_MENUITEM_PROP_LABEL, _("No date yet..."));
+		dbusmenu_menuitem_property_set     (date, DBUSMENU_MENUITEM_PROP_LABEL, _("No date yet…"));
 		dbusmenu_menuitem_property_set_bool(date, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
 		dbusmenu_menuitem_child_append(root, date);
 
@@ -1093,45 +1109,47 @@ build_menus (DbusmenuMenuitem * root)
 
 		g_idle_add(check_for_calendar, NULL);
 	}
+
+	if (!get_greeter_mode ()) {
+		locations_separator = dbusmenu_menuitem_new();
+		dbusmenu_menuitem_property_set(locations_separator, DBUSMENU_MENUITEM_PROP_TYPE, DBUSMENU_CLIENT_TYPES_SEPARATOR);
+		dbusmenu_menuitem_property_set_bool (locations_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
+		dbusmenu_menuitem_child_append(root, locations_separator);
+
+		geo_location = dbusmenu_menuitem_new();
+		dbusmenu_menuitem_property_set      (geo_location, DBUSMENU_MENUITEM_PROP_TYPE, TIMEZONE_MENUITEM_TYPE);
+		set_current_timezone_label (geo_location, "");
+		dbusmenu_menuitem_property_set_bool (geo_location, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
+		dbusmenu_menuitem_property_set_bool (geo_location, DBUSMENU_MENUITEM_PROP_VISIBLE, TRUE);
+		g_signal_connect(G_OBJECT(geo_location), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(quick_set_tz), NULL);
+		dbusmenu_menuitem_child_append(root, geo_location);
+
+		current_location = dbusmenu_menuitem_new();
+		dbusmenu_menuitem_property_set      (current_location, DBUSMENU_MENUITEM_PROP_TYPE, TIMEZONE_MENUITEM_TYPE);
+		set_current_timezone_label (current_location, "");
+		dbusmenu_menuitem_property_set_bool (current_location, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
+		dbusmenu_menuitem_property_set_bool (current_location, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
+		dbusmenu_menuitem_child_append(root, current_location);
 	
-	locations_separator = dbusmenu_menuitem_new();
-	dbusmenu_menuitem_property_set(locations_separator, DBUSMENU_MENUITEM_PROP_TYPE, DBUSMENU_CLIENT_TYPES_SEPARATOR);
-	dbusmenu_menuitem_property_set_bool (locations_separator, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-	dbusmenu_menuitem_child_append(root, locations_separator);
-
-	geo_location = dbusmenu_menuitem_new();
-	dbusmenu_menuitem_property_set      (geo_location, DBUSMENU_MENUITEM_PROP_TYPE, TIMEZONE_MENUITEM_TYPE);
-	set_current_timezone_label (geo_location, "");
-	dbusmenu_menuitem_property_set_bool (geo_location, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
-	dbusmenu_menuitem_property_set_bool (geo_location, DBUSMENU_MENUITEM_PROP_VISIBLE, TRUE);
-	g_signal_connect(G_OBJECT(geo_location), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(quick_set_tz), NULL);
-	dbusmenu_menuitem_child_append(root, geo_location);
-
-	current_location = dbusmenu_menuitem_new();
-	dbusmenu_menuitem_property_set      (current_location, DBUSMENU_MENUITEM_PROP_TYPE, TIMEZONE_MENUITEM_TYPE);
-	set_current_timezone_label (current_location, "");
-	dbusmenu_menuitem_property_set_bool (current_location, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
-	dbusmenu_menuitem_property_set_bool (current_location, DBUSMENU_MENUITEM_PROP_VISIBLE, FALSE);
-	dbusmenu_menuitem_child_append(root, current_location);
+		check_timezone_sync();
 	
-	check_timezone_sync();
-	
-	g_signal_connect (conf, "changed::" SETTINGS_SHOW_LOCATIONS_S, G_CALLBACK (show_locations_changed), NULL);
-	g_signal_connect (conf, "changed::" SETTINGS_LOCATIONS_S, G_CALLBACK (show_locations_changed), NULL);
-	g_signal_connect (conf, "changed::" SETTINGS_SHOW_EVENTS_S, G_CALLBACK (show_events_changed), NULL);
-	g_signal_connect (conf, "changed::" SETTINGS_TIME_FORMAT_S, G_CALLBACK (time_format_changed), NULL);
+		g_signal_connect (conf, "changed::" SETTINGS_SHOW_LOCATIONS_S, G_CALLBACK (show_locations_changed), NULL);
+		g_signal_connect (conf, "changed::" SETTINGS_LOCATIONS_S, G_CALLBACK (show_locations_changed), NULL);
+		g_signal_connect (conf, "changed::" SETTINGS_SHOW_EVENTS_S, G_CALLBACK (show_events_changed), NULL);
+		g_signal_connect (conf, "changed::" SETTINGS_TIME_FORMAT_S, G_CALLBACK (time_format_changed), NULL);
 
-	DbusmenuMenuitem * separator = dbusmenu_menuitem_new();
-	dbusmenu_menuitem_property_set(separator, DBUSMENU_MENUITEM_PROP_TYPE, DBUSMENU_CLIENT_TYPES_SEPARATOR);
-	dbusmenu_menuitem_child_append(root, separator);
+		DbusmenuMenuitem * separator = dbusmenu_menuitem_new();
+		dbusmenu_menuitem_property_set(separator, DBUSMENU_MENUITEM_PROP_TYPE, DBUSMENU_CLIENT_TYPES_SEPARATOR);
+		dbusmenu_menuitem_child_append(root, separator);
 
-	settings = dbusmenu_menuitem_new();
-	dbusmenu_menuitem_property_set     (settings, DBUSMENU_MENUITEM_PROP_LABEL, _("Time & Date Settings..."));
-	/* insensitive until we check for available apps */
-	dbusmenu_menuitem_property_set_bool(settings, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
-	g_signal_connect(G_OBJECT(settings), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(activate_cb), "indicator-datetime-preferences");
-	dbusmenu_menuitem_child_append(root, settings);
-	g_idle_add(check_for_timeadmin, NULL);
+		settings = dbusmenu_menuitem_new();
+		dbusmenu_menuitem_property_set     (settings, DBUSMENU_MENUITEM_PROP_LABEL, _("Time & Date Settings…"));
+		/* insensitive until we check for available apps */
+		dbusmenu_menuitem_property_set_bool(settings, DBUSMENU_MENUITEM_PROP_ENABLED, FALSE);
+		g_signal_connect(G_OBJECT(settings), DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED, G_CALLBACK(activate_cb), "gnome-control-center indicator-datetime");
+		dbusmenu_menuitem_child_append(root, settings);
+		g_idle_add(check_for_timeadmin, NULL);
+	}
 
 	return;
 }
@@ -1412,6 +1430,14 @@ geo_create_client (GeoclueMaster * master, GeoclueMasterClient * client, gchar *
 	return;
 }
 
+static gboolean
+get_greeter_mode (void)
+{
+  const gchar *var;
+  var = g_getenv("INDICATOR_GREETER_MODE");
+  return (g_strcmp0(var, "1") == 0);
+}
+
 /* Repsonds to the service object saying it's time to shutdown.
    It stops the mainloop. */
 static void 
