@@ -41,9 +41,9 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <libindicator/indicator-service-manager.h>
 
 /* DBusMenu */
-#include <libdbusmenu-gtk/menu.h>
+#include <libdbusmenu-gtk3/menu.h>
 #include <libido/libido.h>
-#include <libdbusmenu-gtk/menuitem.h>
+#include <libdbusmenu-gtk3/menuitem.h>
 
 #include "utils.h"
 #include "dbus-shared.h"
@@ -138,9 +138,6 @@ struct _indicator_item_t {
 #define PROP_SHOW_WEEK_NUMBERS_S        "show-week-numbers"
 #define PROP_SHOW_CALENDAR_S            "show-calendar"
 
-#define INDICATOR_DATETIME_GET_PRIVATE(o) \
-(G_TYPE_INSTANCE_GET_PRIVATE ((o), INDICATOR_DATETIME_TYPE, IndicatorDatetimePrivate))
-
 enum {
 	STRFTIME_MASK_NONE    = 0,      /* Hours or minutes as we always test those */
 	STRFTIME_MASK_SECONDS = 1 << 0, /* Seconds count */
@@ -153,7 +150,7 @@ enum {
 	STRFTIME_MASK_ALL     = (STRFTIME_MASK_SECONDS | STRFTIME_MASK_AMPM | STRFTIME_MASK_WEEK | STRFTIME_MASK_DAY | STRFTIME_MASK_MONTH | STRFTIME_MASK_YEAR)
 };
 
-GType indicator_datetime_get_type (void);
+GType indicator_datetime_get_type (void) G_GNUC_CONST;
 
 static void indicator_datetime_class_init (IndicatorDatetimeClass *klass);
 static void indicator_datetime_init       (IndicatorDatetime *self);
@@ -215,7 +212,7 @@ indicator_datetime_class_init (IndicatorDatetimeClass *klass)
 	                                 PROP_TIME_FORMAT,
 	                                 g_param_spec_int(PROP_TIME_FORMAT_S,
 	                                                  "A choice of which format should be used on the panel",
-	                                                  "Chooses between letting the locale choose the time, 12-hour time, 24-time or using the custom string passed to strftime().",
+	                                                  "Chooses between letting the locale choose the time, 12-hour time, 24-time or using the custom string passed to g_date_time_format().",
 	                                                  SETTINGS_TIME_LOCALE, /* min */
 	                                                  SETTINGS_TIME_CUSTOM, /* max */
 	                                                  SETTINGS_TIME_LOCALE, /* default */
@@ -245,7 +242,7 @@ indicator_datetime_class_init (IndicatorDatetimeClass *klass)
 	                                 PROP_CUSTOM_TIME_FORMAT,
 	                                 g_param_spec_string(PROP_CUSTOM_TIME_FORMAT_S,
 	                                                     "The format that is used to show the time on the panel.",
-	                                                     "A format string in the form used to pass to strftime to make a string for displaying on the panel.",
+	                                                     "A format string in the form used to pass to g_date_time_format() to make a string for displaying on the panel.",
 	                                                     DEFAULT_TIME_FORMAT,
 	                                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
@@ -270,25 +267,18 @@ static void
 menu_visible_notfy_cb(GtkWidget * menu, G_GNUC_UNUSED GParamSpec *pspec, gpointer user_data)
 {
 	IndicatorDatetime * self = INDICATOR_DATETIME(user_data);
-	g_debug("notify visible signal recieved");
-	
-	// we should only react if we're currently visible
-	gboolean visible;
-	g_object_get(G_OBJECT(menu), "visible", &visible, NULL);
-	if (visible) return;
-	g_debug("notify visible menu hidden, resetting date");
-	
-	time_t curtime;
-	
-	time(&curtime);
-  	struct tm *today = localtime(&curtime);
-  	int y = today->tm_year;
-  	int m = today->tm_mon;
-  	int d = today->tm_mday;
-  	
+	GDateTime *datetime;
+	gint y, m, d;
+
+	g_debug("notify visible signal received");
+
+	datetime = g_date_time_new_now_local ();
+	g_date_time_get_ymd (datetime, &y, &m, &d);
+	g_date_time_unref (datetime);
+
   	// Set the calendar to todays date
-	ido_calendar_menu_item_set_date (self->priv->ido_calendar, y+1900, m, d);
-	
+	ido_calendar_menu_item_set_date (self->priv->ido_calendar, y, m-1, d);
+
 	// Make sure the day-selected signal is sent so the menu updates - may duplicate
 	/*GVariant *variant = g_variant_new_uint32((guint)curtime);
 	guint timestamp = (guint)time(NULL);
@@ -298,7 +288,9 @@ menu_visible_notfy_cb(GtkWidget * menu, G_GNUC_UNUSED GParamSpec *pspec, gpointe
 static void
 indicator_datetime_init (IndicatorDatetime *self)
 {
-	self->priv = INDICATOR_DATETIME_GET_PRIVATE(self);
+	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+                                                  INDICATOR_DATETIME_TYPE,
+                                                  IndicatorDatetimePrivate);
 
 	self->priv->label = NULL;
 	self->priv->timer = 0;
@@ -407,11 +399,10 @@ service_proxy_cb (GObject * object, GAsyncResult * res, gpointer user_data)
 
 	IndicatorDatetime * self = INDICATOR_DATETIME(user_data);
 	g_return_if_fail(self != NULL);
+	IndicatorDatetimePrivate * priv = self->priv;
 
 	GDBusProxy * proxy = g_dbus_proxy_new_for_bus_finish(res, &error);
 
-	IndicatorDatetimePrivate * priv = INDICATOR_DATETIME_GET_PRIVATE(self);
-
 	if (priv->service_proxy_cancel != NULL) {
 		g_object_unref(priv->service_proxy_cancel);
 		priv->service_proxy_cancel = NULL;
@@ -544,7 +535,8 @@ bind_enum_get (GValue * value, GVariant * variant, gpointer user_data)
 static void
 timezone_update_all_labels (IndicatorDatetime * self)
 {
-	IndicatorDatetimePrivate *priv = INDICATOR_DATETIME_GET_PRIVATE(self);
+	IndicatorDatetimePrivate *priv = self->priv;
+
 	g_list_foreach(priv->timezone_items, (GFunc)timezone_update_labels, NULL);
 }
 
@@ -761,11 +753,18 @@ set_label_to_time_in_zone (IndicatorDatetime * self, GtkLabel * label,
                            GTimeZone * tz, const gchar * format,
                            GDateTime ** datetime)
 {
+	gboolean unref_tz = FALSE;
+	if (tz == NULL) {
+		gchar * zone = read_timezone ();
+		if (zone == NULL)
+			return;
+		tz = g_time_zone_new(zone);
+		unref_tz = TRUE;
+		g_free (zone);
+	}
+
 	GDateTime * datetime_now;
-	if (tz == NULL)
-		datetime_now = g_date_time_new_now_local();
-	else
-		datetime_now = g_date_time_new_now(tz);
+	datetime_now = g_date_time_new_now(tz);
 
 	gchar * timestr;
 	if (format == NULL) {
@@ -775,6 +774,12 @@ set_label_to_time_in_zone (IndicatorDatetime * self, GtkLabel * label,
 	}
 	else {
 		timestr = g_date_time_format(datetime_now, format);
+		if (timestr == NULL) {
+			g_warning ("The custom date format is not valid, check the\n"
+			           "g_date_time_format() documentation for the supported\n"
+			           "format specifiers ");
+			timestr = g_strdup ("Date format not supported");
+		}
 	}
 
 	gboolean use_markup = FALSE;
@@ -793,6 +798,9 @@ set_label_to_time_in_zone (IndicatorDatetime * self, GtkLabel * label,
 	else
 		g_date_time_unref(datetime_now);
 
+	if (unref_tz)
+		g_time_zone_unref(tz);
+
 	return;
 }
 
@@ -819,11 +827,13 @@ update_label (IndicatorDatetime * io, GDateTime ** datetime)
 static void
 update_time (IndicatorDatetime * self)
 {
-	GDateTime * dt;
+	GDateTime * dt = NULL;
 	update_label(self, &dt);
 	timezone_update_all_labels(self);
-	setup_timer(self, dt);
-	g_date_time_unref(dt);
+	if (dt != NULL) {
+		setup_timer(self, dt);
+		g_date_time_unref(dt);
+  }
 	return;
 }
 
@@ -847,11 +857,13 @@ timer_func (gpointer user_data)
 {
 	IndicatorDatetime * self = INDICATOR_DATETIME(user_data);
 	self->priv->timer = 0;
-	GDateTime * dt;
+	GDateTime * dt = NULL;
 	update_label(self, &dt);
 	timezone_update_all_labels(self);
-	setup_timer(self, dt);
-	g_date_time_unref(dt);
+	if (dt != NULL) {
+		setup_timer(self, dt);
+		g_date_time_unref(dt);
+  }
 	return FALSE;
 }
 
@@ -1077,15 +1089,17 @@ guess_label_size (IndicatorDatetime * self)
 	GArray * timevals = g_array_new(FALSE, TRUE, sizeof(struct tm));
 	build_timeval_array(timevals, posibilitymask);
 
-	g_debug("Checking against %d posible times", timevals->len);
+	g_debug("Checking against %d possible times", timevals->len);
 	gint check_time;
 	for (check_time = 0; check_time < timevals->len; check_time++) {
-		gchar longstr[256];
-		strftime(longstr, 256, self->priv->time_string, &(g_array_index(timevals, struct tm, check_time)));
+		struct tm * timeval = &g_array_index(timevals, struct tm, check_time);
+		GDateTime * dt = g_date_time_new_local(timeval->tm_year, timeval->tm_mon, timeval->tm_mday, timeval->tm_hour, timeval->tm_min, timeval->tm_sec);
+		gchar * timestr = g_date_time_format(dt, self->priv->time_string);
 
-		gchar * utf8 = g_locale_to_utf8(longstr, -1, NULL, NULL, NULL);
-		gint length = measure_string(style, context, utf8);
-		g_free(utf8);
+		gint length = measure_string(style, context, timestr);
+
+		g_free(timestr);
+		g_date_time_unref(dt);
 
 		if (length > *max_width) {
 			*max_width = length;
@@ -1285,6 +1299,13 @@ new_appointment_item (DbusmenuMenuitem * newitem, DbusmenuMenuitem * parent, Dbu
 	/* Label, probably a username, chat room or mailbox name */
 	mi_data->label = gtk_label_new(dbusmenu_menuitem_property_get(newitem, APPOINTMENT_MENUITEM_PROP_LABEL));
 	gtk_misc_set_alignment(GTK_MISC(mi_data->label), 0.0, 0.5);
+	
+	GtkStyle * style = gtk_widget_get_style(GTK_WIDGET(mi_data->label));
+	PangoContext * context = gtk_widget_get_pango_context(GTK_WIDGET(mi_data->label));
+	gint length = measure_string(style, context, "MMMMMMMMMMMMMMM"); // 15 char wide string max
+	gtk_widget_set_size_request(GTK_WIDGET(mi_data->label), length, -1); // Set the min size in pixels
+	
+	gtk_label_set_ellipsize(GTK_LABEL(mi_data->label), PANGO_ELLIPSIZE_END);
 	gtk_box_pack_start(GTK_BOX(hbox), mi_data->label, TRUE, TRUE, 0);
 	gtk_widget_show(mi_data->label);
 
@@ -1372,8 +1393,7 @@ new_calendar_item (DbusmenuMenuitem * newitem,
 	/* Note: not checking parent, it's reasonable for it to be NULL */
 
 	IndicatorDatetime *self = INDICATOR_DATETIME(user_data);
-	self->priv = INDICATOR_DATETIME_GET_PRIVATE(self);
-	
+
 	IdoCalendarMenuItem *ido = IDO_CALENDAR_MENU_ITEM (ido_calendar_menu_item_new ());
 	self->priv->ido_calendar = ido;
 	
@@ -1417,7 +1437,9 @@ timezone_toggled_cb (GtkCheckMenuItem *checkmenuitem, DbusmenuMenuitem * dbusite
 static void
 timezone_destroyed_cb (indicator_item_t * mi_data, DbusmenuMenuitem * dbusitem)
 {
-	IndicatorDatetimePrivate *priv = INDICATOR_DATETIME_GET_PRIVATE(mi_data->self);
+	IndicatorDatetime *self = INDICATOR_DATETIME (mi_data->self);
+	IndicatorDatetimePrivate *priv = self->priv;
+
 	priv->timezone_items = g_list_remove(priv->timezone_items, mi_data);
 	g_signal_handlers_disconnect_by_func(G_OBJECT(mi_data->gmi), G_CALLBACK(timezone_toggled_cb), dbusitem);
 	g_free(mi_data);
@@ -1435,7 +1457,7 @@ new_timezone_item(DbusmenuMenuitem * newitem,
 	/* Note: not checking parent, it's reasonable for it to be NULL */
 
 	IndicatorDatetime * self = INDICATOR_DATETIME(user_data);
-	IndicatorDatetimePrivate *priv = INDICATOR_DATETIME_GET_PRIVATE(self);
+	IndicatorDatetimePrivate *priv = self->priv;
 
 	// Menu item with a radio button and a right aligned time
 	indicator_item_t * mi_data = g_new0(indicator_item_t, 1);
